'use strict';

var filter = require('lodash/filter');
var TreeIndex = require('../util/TreeIndex');
var DocumentIndex = require('./DocumentIndex');

function AnchorIndex(doc) {
  this.doc = doc;
  this.byPath = new TreeIndex.Arrays();
  this.byId = {};
}

AnchorIndex.Prototype = function() {

  this.select = function(node) {
    return (node._isContainerAnnotation);
  };

  this.reset = function(data) {
    this.byPath.clear();
    this.byId = {};
    this._initialize(data);
  };

  this.get = function(path, containerName) {
    var anchors = this.byPath.getAll(path);
    if (containerName) {
      return filter(anchors, function(anchor) {
        return (anchor.containerId === containerName);
      });
    } else {
      // return a copy of the array
      return anchors.slice(0);
    }
  };

  this.create = function(containerAnno) {
    var startAnchor = containerAnno.getStartAnchor();
    var endAnchor = containerAnno.getEndAnchor();
    this.byPath.add(startAnchor.path, startAnchor);
    this.byPath.add(endAnchor.path, endAnchor);
    this.byId[containerAnno.id] = containerAnno;
  };

  this.delete = function(containerAnno) {
    var startAnchor = containerAnno.getStartAnchor();
    var endAnchor = containerAnno.getEndAnchor();
    this.byPath.remove(startAnchor.path, startAnchor);
    this.byPath.remove(endAnchor.path, endAnchor);
    delete this.byId[containerAnno.id];
  };

  this.update = function(node, path, newValue, oldValue) {
    if (this.select(node)) {
      var anchor = null;
      if (path[1] === 'startPath') {
        anchor = node.getStartAnchor();
      } else if (path[1] === 'endPath') {
        anchor = node.getEndAnchor();
      } else {
        return;
      }
      this.byPath.remove(oldValue, anchor);
      this.byPath.add(anchor.path, anchor);
    }
  };

};

DocumentIndex.extend(AnchorIndex);

module.exports = AnchorIndex;
